@@ -223,7 +223,6 @@ static void renderSelectedNodes(Agraph_t * g)
     Agsym_t* l_color_attr = GG_nodelabelcolor(g);
     glCompColor c;
     int defaultNodeShape;
-    float nodeSize;
 
     glCompColorxlate(&c,agxget(g,l_color_attr));
 
@@ -245,14 +244,14 @@ static void renderSelectedNodes(Agraph_t * g)
     {
 	if(!ND_selected(v))
 	    continue;
-	glColor4f(view->selectedNodeColor.R, view->selectedNodeColor.G,view->selectedNodeColor.B, view->selectedNodeColor.A);
+	glColor4d(view->selectedNodeColor.R, view->selectedNodeColor.G,view->selectedNodeColor.B, view->selectedNodeColor.A);
 	pos = ND_A(v);
-	nodeSize = ND_size(v);
+	const double nodeSize = ND_size(v);
 
 	if (defaultNodeShape == 0) 
 	    glVertex3f(pos.x, pos.y, pos.z + 0.001f);
 	else if (defaultNodeShape == 1) 
-	    drawCircle(pos.x, pos.y, nodeSize, pos.z + 0.001f);
+	    drawCircle(pos.x, pos.y, nodeSize, pos.z + 0.001);
     }
     if(defaultNodeShape==0)
 	glEnd();
@@ -263,7 +262,7 @@ static void renderSelectedNodes(Agraph_t * g)
 	if (ND_printLabel(v)==1)
 	{
 	    pos = ND_A(v);
-	    glColor4f(c.R, c.G,c.B, c.A);
+	    glColor4d(c.R, c.G,c.B, c.A);
 	    glprintfglut(view->glutfont, pos.x, pos.y, pos.z + 0.002f, labelOf(g, v));
 	}
     }
@@ -314,9 +313,9 @@ static void renderNodes(Agraph_t * g)
 	{
 	    ND_selected(v) = 1;
 	}
-	glColor4f(c.R,c.G,c.B,c.A);	    
+	glColor4d(c.R,c.G,c.B,c.A);	    
 	const glCompPoint pos = getPointFromStr(agxget(v, pos_attr));
-	float nodeSize = l_float(v, size_attr, 0);
+	double nodeSize = l_float(v, size_attr, 0);
 
 	ND_A(v) = pos;
 
@@ -582,7 +581,7 @@ static void edge_seg (Agraph_t* g, Agedge_t* e, glCompColor c)
 {
     Agsym_t* pos_attr = GN_pos(g);
 
-    glColor4f(c.R,c.G,c.B,c.A);	   
+    glColor4d(c.R,c.G,c.B,c.A);	   
     // tail position
     const glCompPoint posT = getPointFromStr(agxget(agtail(e), pos_attr));
     // head position
@@ -596,7 +595,7 @@ static void edge_spline (Agraph_t* g, Agedge_t* e, glCompColor c)
 {
     Agsym_t* pos_attr_e = GE_pos(g);
 
-    glColor4f(c.R,c.G,c.B,c.A);	   
+    glColor4d(c.R,c.G,c.B,c.A);	   
     xdot *const x = makeXDotSpline(agxget(e, pos_attr_e));
     if (x) {
 	draw_xdot(x,0);
@@ -639,7 +638,7 @@ static void renderNodeLabels(Agraph_t * g)
 	    continue;
 
 	pos = ND_A(v);
-	glColor4f(c.R,c.G,c.B,c.A);
+	glColor4d(c.R,c.G,c.B,c.A);
 	if(!data_attr)
             glprintfglut(view->glutfont,pos.x,pos.y,pos.z,agnameof(v));
 	else
@@ -672,7 +671,7 @@ static void renderEdgeLabels(Agraph_t * g)
 
 	    posT = ED_posTail(e);
 	    posH = ED_posHead(e);
-	    glColor4f(c.R,c.G,c.B,c.A);
+	    glColor4d(c.R,c.G,c.B,c.A);
 	    float x = posH.x + (posT.x - posH.x) / 2;
 	    float y = posH.y + (posT.y - posH.y) / 2;
 	    float z = posH.z + (posT.z - posH.z) / 2;
@@ -826,19 +825,19 @@ void renderSmGraph(topview* t)
         glCallList(t->cache.seledge_id);
         if(view->drawedgelabels)
 	{
-	    if(view->zoom*-1 <	t->fitin_zoom /(float)view->labelnumberofnodes*-1) 
+	    if (view->zoom * -1 < t->fitin_zoom / view->labelnumberofnodes * -1)
 		    glCallList(t->cache.edgelabel_id);
 
 	}
     }
     if(view->drawnodes)
     {
-	glPointSize(view->nodeScale*t->fitin_zoom/view->zoom);
+	glPointSize((float)(view->nodeScale * t->fitin_zoom / view->zoom));
 	glCallList(t->cache.node_id);
         glCallList(t->cache.selnode_id);
         if(view->drawnodelabels)
 	{
-	    if(view->zoom*-1 <	t->fitin_zoom /(float)view->labelnumberofnodes*-1) 
+	    if (view->zoom * -1 < t->fitin_zoom / view->labelnumberofnodes * -1)
 		glCallList(t->cache.nodelabel_id);
 	}
     }
