@@ -35,7 +35,11 @@
 #include <util/alloc.h>
 #include <util/exit.h>
 #include <util/gv_ctype.h>
+#include <util/gv_find_me.h>
+#include <util/gv_fopen.h>
 #include <util/list.h>
+#include <util/path.h>
+#include <util/strview.h>
 #include <util/unreachable.h>
 
 static char *Info[] = {
@@ -53,6 +57,7 @@ static const char *usage =
    -o <ofile> - write output to <ofile>; stdout by default\n\
    -n         - no read-ahead of input graphs\n\
    -q         - turn off warning messages\n\
+   -v         - enable verbose messages\n\
    -V         - print version info\n\
    -?         - print usage info\n\
 If no files are specified, stdin is used\n";
@@ -65,22 +70,20 @@ typedef struct {
   compflags_t compflags;
   int readAhead;
   char **inFiles;
-  int argc;
-  char **argv;
+  strviews_t args;
   int state; /* > 0 : continue; <= 0 finish */
   int verbose;
 } options;
 
 static FILE *openOut(char *name) {
-  FILE *outs = fopen(name, "w");
+  FILE *const outs = gv_fopen(name, "w");
   if (outs == 0) {
     error(ERROR_ERROR, "could not open %s for writing", name);
   }
   return outs;
 }
 
-/* gettok:
- * Tokenize a string. Tokens consist of either a non-empty string
+/* Tokenize a string. Tokens consist of either a non-empty string
  * of non-space characters, or all characters between a pair of
  * single or double quotes. As usual, we map
  *   \c -> c
@@ -99,12 +102,12 @@ static char *gettok(char **sp) {
 
   while (gv_isspace(*rs))
     rs++;
-  if ((c = *rs) == '\0')
+  if (*rs == '\0')
     return NULL;
   while ((c = *rs)) {
-    if (q && (q == c)) { /* end quote */
+    if (q && q == c) { /* end quote */
       q = '\0';
-    } else if (!q && ((c == '"') || (c == '\''))) {
+    } else if (!q && (c == '"' || c == '\'')) {
       q = c;
     } else if (c == '\\') {
       rs++;
@@ -131,48 +134,22 @@ static char *gettok(char **sp) {
   return s;
 }
 
-#define NUM_ARGS 100
-
-/* parseArgs:
- * Split s into whitespace separated tokens, allowing quotes.
+/* Split s into whitespace separated tokens, allowing quotes.
  * Append tokens to argument list and return new number of arguments.
- * argc is the current number of arguments, with the arguments
- * stored in *argv.
+ *
+ * @param arg [inout] The current arguments
  */
-static int parseArgs(char *s, int argc, char ***argv) {
-  int i, cnt = 0;
-  char *args[NUM_ARGS];
+static void parseArgs(char *s, strviews_t *arg) {
   char *t;
-  char **av;
-
-  assert(argc >= 0);
 
   while ((t = gettok(&s))) {
-    if (cnt == NUM_ARGS) {
-      error(ERROR_WARNING,
-            "at most %d arguments allowed per -a flag - ignoring rest",
-            NUM_ARGS);
-      break;
-    }
-    args[cnt++] = t;
+    LIST_APPEND(arg, strview(t, '\0'));
   }
-
-  if (cnt) {
-    int oldcnt = argc;
-    argc = oldcnt + cnt;
-    av = gv_recalloc(*argv, (size_t)oldcnt, (size_t)argc, sizeof(char *));
-    for (i = 0; i < cnt; i++)
-      av[oldcnt + i] = gv_strdup(args[i]);
-    *argv = av;
-  }
-  return argc;
 }
 
 #if defined(_WIN32) && !defined(__MINGW32__)
-#define PATHSEP '\\'
 #define LISTSEP ';'
 #else
-#define PATHSEP '/'
 #define LISTSEP ':'
 #endif
 
@@ -182,8 +159,53 @@ static char *concat(char *pfx, char *sfx) {
   return agxbdisown(&sp);
 }
 
-/* resolve:
- * Translate -f arg parameter into a pathname.
+/// get gvpr’s default paths to search for scripts
+///
+/// The content returned is a list of paths separated by the platform’s native
+/// `$PATH` separator, either ':' or ';'. The caller is responsible for freeing
+/// the returned string.
+///
+/// @return Search path for scripts or `NULL` on failure
+static char *dflt_gvprpath(void) {
+
+  // find our containing executable
+  char *const exe = gv_find_me();
+  if (exe == NULL) {
+    return NULL;
+  }
+
+  // assume it is of the form …/bin/foo[.exe] and construct
+  // .:…/share/graphviz/gvpr
+
+  char *slash = strrchr(exe, PATH_SEPARATOR);
+  if (slash == NULL) {
+    free(exe);
+    return NULL;
+  }
+
+  *slash = '\0';
+  slash = strrchr(exe, PATH_SEPARATOR);
+  if (slash == NULL) {
+    free(exe);
+    return NULL;
+  }
+
+  *slash = '\0';
+  const size_t share_len =
+      strlen(".:") + strlen(exe) + strlen("/share/graphviz/gvpr") + 1;
+  char *const share = malloc(share_len);
+  if (share == NULL) {
+    free(exe);
+    return NULL;
+  }
+  snprintf(share, share_len, ".%c%s%cshare%cgraphviz%cgvpr", LISTSEP, exe,
+           PATH_SEPARATOR, PATH_SEPARATOR, PATH_SEPARATOR);
+  free(exe);
+
+  return share;
+}
+
+/* Translate -f arg parameter into a pathname.
  * If arg contains '/', return arg.
  * Else search directories in GVPRPATH for arg.
  * Return NULL on error.
@@ -197,20 +219,27 @@ static char *resolve(char *arg, int verbose) {
   char *pathp = NULL;
   size_t sz;
 
-  if (strchr(arg, PATHSEP))
+  if (strchr(arg, PATH_SEPARATOR))
     return gv_strdup(arg);
 
+  char *const dflt = dflt_gvprpath();
+  if (dflt == NULL) {
+    error(ERROR_ERROR, "Could not determine DFLT_GVPRPATH");
+  }
+
   path = getenv("GVPRPATH");
   if (!path)
     path = getenv("GPRPATH"); // deprecated
-  if (path && (c = *path)) {
+  if (dflt == NULL) {
+    // do not use `dflt` at all
+  } else if (path && (c = *path)) {
     if (c == LISTSEP) {
-      pathp = path = concat(DFLT_GVPRPATH, path);
-    } else if ((c = path[strlen(path) - 1]) == LISTSEP) {
-      pathp = path = concat(path, DFLT_GVPRPATH);
+      pathp = path = concat(dflt, path);
+    } else if (path[strlen(path) - 1] == LISTSEP) {
+      pathp = path = concat(path, dflt);
     }
   } else
-    path = DFLT_GVPRPATH;
+    path = dflt;
   if (verbose)
     fprintf(stderr, "PATH: %s\n", path);
   agxbuf fp = {0};
@@ -229,7 +258,7 @@ static char *resolve(char *arg, int verbose) {
       sz = agxbput(&fp, path);
       path += sz;
     }
-    agxbprint(&fp, "%c%s", PATHSEP, arg);
+    agxbprint(&fp, "%c%s", PATH_SEPARATOR, arg);
     s = agxbuse(&fp);
 
     if (access(s, R_OK) == 0) {
@@ -237,13 +266,16 @@ static char *resolve(char *arg, int verbose) {
     }
   }
 
+  free(dflt);
+
   if (!fname)
     error(ERROR_ERROR, "Could not find file \"%s\" in GVPRPATH", arg);
 
   agxbfree(&fp);
   free(pathp);
   if (verbose)
-    fprintf(stderr, "file %s resolved to %s\n", arg, fname);
+    fprintf(stderr, "file %s resolved to %s\n", arg,
+            fname == NULL ? "<null>" : fname);
   return fname;
 }
 
@@ -267,8 +299,7 @@ static char *getOptarg(int c, char **argp, int *argip, int argc, char **argv) {
   return rv;
 }
 
-/* doFlags:
- * Process a command-line argument starting with a '-'.
+/* Process a command-line argument starting with a '-'.
  * argi is the index of the next available item in argv[].
  * argc has its usual meaning.
  *
@@ -303,7 +334,7 @@ static int doFlags(char *arg, int argi, int argc, char **argv, options *opts) {
       break;
     case 'a':
       if ((optarg = getOptarg(c, &arg, &argi, argc, argv))) {
-        opts->argc = parseArgs(optarg, opts->argc, &(opts->argv));
+        parseArgs(optarg, &opts->args);
       } else
         return -1;
       break;
@@ -321,7 +352,6 @@ static int doFlags(char *arg, int argi, int argc, char **argv, options *opts) {
     case 'V':
       fprintf(stderr, "%s version %s (%s)\n", Info[0], Info[1], Info[2]);
       return 0;
-      break;
     case '?':
       if (optopt == '\0' || optopt == '?')
         fprintf(stderr, "Usage: gvpr%s", usage);
@@ -329,7 +359,6 @@ static int doFlags(char *arg, int argi, int argc, char **argv, options *opts) {
         error(ERROR_USAGE | ERROR_WARNING, "%s", usage);
       }
       return 0;
-      break;
     default:
       error(ERROR_WARNING, "option -%c unrecognized", c);
       break;
@@ -344,16 +373,10 @@ static void freeOpts(options opts) {
   free(opts.inFiles);
   if (opts.useFile)
     free(opts.program);
-  for (int i = 0; i < opts.argc; i++)
-    free(opts.argv[i]);
-  free(opts.argv);
+  LIST_FREE(&opts.args);
 }
 
-DEFINE_LIST(strs, char *)
-
-/* scanArgs:
- * Parse command line options.
- */
+/// parse command line options
 static options scanArgs(int argc, char **argv) {
   char *arg;
   options opts = {0};
@@ -364,7 +387,7 @@ static options scanArgs(int argc, char **argv) {
   setErrorId(opts.cmdName);
   opts.verbose = 0;
 
-  strs_t input_filenames = {0};
+  LIST(char *) input_filenames = {0};
 
   /* loop over arguments */
   for (int i = 1; i < argc;) {
@@ -376,24 +399,24 @@ static options scanArgs(int argc, char **argv) {
         goto opts_done;
       }
     } else if (arg)
-      strs_append(&input_filenames, arg);
+      LIST_APPEND(&input_filenames, arg);
   }
 
   /* Handle additional semantics */
   if (opts.useFile == 0) {
-    if (strs_is_empty(&input_filenames)) {
+    if (LIST_IS_EMPTY(&input_filenames)) {
       error(ERROR_ERROR, "No program supplied via argument or -f option");
       opts.state = -1;
     } else {
-      opts.program = strs_pop_front(&input_filenames);
+      opts.program = LIST_POP_FRONT(&input_filenames);
     }
   }
-  if (strs_is_empty(&input_filenames)) {
+  if (LIST_IS_EMPTY(&input_filenames)) {
     opts.inFiles = 0;
-    strs_free(&input_filenames);
+    LIST_FREE(&input_filenames);
   } else {
-    strs_append(&input_filenames, NULL);
-    opts.inFiles = strs_detach(&input_filenames);
+    LIST_APPEND(&input_filenames, NULL);
+    LIST_DETACH(&input_filenames, &opts.inFiles, NULL);
   }
 
   if (!opts.outFile)
@@ -403,7 +426,7 @@ opts_done:
   if (opts.state <= 0) {
     if (opts.state < 0)
       error(ERROR_USAGE | ERROR_ERROR, "%s", usage);
-    strs_free(&input_filenames);
+    LIST_FREE(&input_filenames);
   }
 
   return opts;
@@ -509,11 +532,9 @@ static trav_fns DFSfns = {agfstedge, agnxtedge, 1, 0};
 static trav_fns FWDfns = {agfstout, agnxtout_, 0, 0};
 static trav_fns REVfns = {agfstin, agnxtin_, 0, 0};
 
-DEFINE_LIST(node_queue, Agnode_t *)
-
 static void travBFS(Gpr_t *state, Expr_t *prog, comp_block *xprog) {
   nodestream nodes;
-  node_queue_t q = {0};
+  LIST(Agnode_t *) q = {0};
   ndata *nd;
   Agnode_t *n;
   Agedge_t *cure;
@@ -527,9 +548,9 @@ static void travBFS(Gpr_t *state, Expr_t *prog, comp_block *xprog) {
     if (MARKED(nd))
       continue;
     PUSH(nd, 0);
-    node_queue_push_back(&q, n);
-    while (!node_queue_is_empty(&q)) {
-      n = node_queue_pop_front(&q);
+    LIST_PUSH_BACK(&q, n);
+    while (!LIST_IS_EMPTY(&q)) {
+      n = LIST_POP_FRONT(&q);
       nd = nData(n);
       MARK(nd);
       POP(nd);
@@ -544,22 +565,20 @@ static void travBFS(Gpr_t *state, Expr_t *prog, comp_block *xprog) {
         if (!evalEdge(state, prog, xprog, cure))
           continue;
         if (!ONSTACK(nd)) {
-          node_queue_push_back(&q, cure->node);
+          LIST_PUSH_BACK(&q, cure->node);
           PUSH(nd, cure);
         }
       }
     }
   }
   state->tvedge = 0;
-  node_queue_free(&q);
+  LIST_FREE(&q);
 }
 
-DEFINE_LIST(edge_stack, Agedge_t *)
-
 static void travDFS(Gpr_t *state, Expr_t *prog, comp_block *xprog,
                     trav_fns *fns) {
   Agnode_t *n;
-  edge_stack_t stk = {0};
+  LIST(Agedge_t *) stk = {0};
   Agnode_t *curn;
   Agedge_t *cure;
   Agedge_t *entry;
@@ -577,7 +596,7 @@ static void travDFS(Gpr_t *state, Expr_t *prog, comp_block *xprog,
     seed.out.node = n;
     seed.in.node = 0;
     curn = n;
-    entry = &(seed.out);
+    entry = &seed.out;
     state->tvedge = cure = 0;
     MARK(nd);
     PUSH(nd, 0);
@@ -605,7 +624,7 @@ static void travDFS(Gpr_t *state, Expr_t *prog, comp_block *xprog,
             evalEdge(state, prog, xprog, cure);
         } else {
           evalEdge(state, prog, xprog, cure);
-          edge_stack_push_back(&stk, entry);
+          LIST_PUSH_BACK(&stk, entry);
           state->tvedge = entry = cure;
           curn = cure->node;
           cure = 0;
@@ -620,8 +639,8 @@ static void travDFS(Gpr_t *state, Expr_t *prog, comp_block *xprog,
         nd = nData(curn);
         POP(nd);
         cure = entry;
-        entry = edge_stack_is_empty(&stk) ? NULL : edge_stack_pop_back(&stk);
-        if (entry == &(seed.out))
+        entry = LIST_IS_EMPTY(&stk) ? NULL : LIST_POP_BACK(&stk);
+        if (entry == &seed.out)
           state->tvedge = 0;
         else
           state->tvedge = entry;
@@ -633,7 +652,7 @@ static void travDFS(Gpr_t *state, Expr_t *prog, comp_block *xprog,
     }
   }
   state->tvedge = 0;
-  edge_stack_free(&stk);
+  LIST_FREE(&stk);
 }
 
 static void travNodes(Gpr_t *state, Expr_t *prog, comp_block *xprog) {
@@ -680,9 +699,7 @@ static void travFlat(Gpr_t *state, Expr_t *prog, comp_block *xprog) {
   }
 }
 
-/* doCleanup:
- * Reset node traversal data
- */
+/// reset node traversal data
 static void doCleanup(Agraph_t *g) {
   Agnode_t *n;
   ndata *nd;
@@ -694,9 +711,7 @@ static void doCleanup(Agraph_t *g) {
   }
 }
 
-/* traverse:
- * return true if traversal requires cleanup
- */
+/// return true if traversal requires cleanup
 static bool traverse(Gpr_t *state, Expr_t *prog, comp_block *bp, bool cleanup) {
   if (!state->target) {
     char *target;
@@ -807,8 +822,7 @@ static bool traverse(Gpr_t *state, Expr_t *prog, comp_block *bp, bool cleanup) {
   return cleanup;
 }
 
-/* addOutputGraph:
- * Append output graph to option struct.
+/* Append output graph to option struct.
  * We know uopts and state->outgraph are non-NULL.
  */
 static void addOutputGraph(Gpr_t *state, gvpropts *uopts) {
@@ -852,8 +866,7 @@ typedef struct {
   options opts;
 } gvpr_state_t;
 
-/* gvexitf:
- * Only used if GV_USE_EXIT not set during exeval.
+/* Only used if GV_USE_EXIT not set during exeval.
  * This implies setjmp/longjmp set up.
  */
 static void gvexitf(void *env, int v) {
@@ -880,8 +893,7 @@ static void gverrorf(Expr_t *handle, Exdisc_t *discipline, int level,
   }
 }
 
-/* gvpr_core:
- * Return 0 on success; non-zero on error.
+/* Return 0 on success; non-zero on error.
  *
  * FIX/TODO:
  *  - close non-source/non-output graphs
@@ -909,14 +921,10 @@ static int gvpr_core(int argc, char *argv[], gvpropts *uopts,
     return 1;
   }
   info.outFile = gs->opts.outFile;
-  info.argc = gs->opts.argc;
-  info.argv = gs->opts.argv;
+  info.args = gs->opts.args;
   info.errf = gverrorf;
-  if (uopts)
-    info.flags = uopts->flags;
-  else
-    info.flags = 0;
-  if ((uopts->flags & GV_USE_EXIT))
+  info.flags = uopts->flags;
+  if (uopts->flags & GV_USE_EXIT)
     info.exitf = 0;
   else
     info.exitf = gvexitf;
@@ -933,7 +941,7 @@ static int gvpr_core(int argc, char *argv[], gvpropts *uopts,
 
   initGPRState(gs->state);
 
-  if ((uopts->flags & GV_USE_OUTGRAPH)) {
+  if (uopts->flags & GV_USE_OUTGRAPH) {
     uopts->outgraphs = 0;
     uopts->n_outgraphs = 0;
   }
@@ -945,7 +953,7 @@ static int gvpr_core(int argc, char *argv[], gvpropts *uopts,
     }
   }
 
-  bool incoreGraphs = uopts && uopts->ingraphs;
+  bool incoreGraphs = uopts->ingraphs;
 
   if (gs->opts.verbose)
     fprintf(stderr, "Parse/compile/init: %.2f secs.\n", gvelapsed_sec());
@@ -955,7 +963,7 @@ static int gvpr_core(int argc, char *argv[], gvpropts *uopts,
 
   /* if program is not null */
   if (gs->xprog->uses_graph) {
-    if (uopts && uopts->ingraphs)
+    if (uopts->ingraphs)
       gs->ing = newIngGraphs(0, uopts->ingraphs, ing_read);
     else
       gs->ing = newIng(0, gs->opts.inFiles, ing_read);
@@ -1008,7 +1016,7 @@ static int gvpr_core(int argc, char *argv[], gvpropts *uopts,
        */
       if (gs->state->outgraph != NULL &&
           (agnnodes(gs->state->outgraph) || gs->opts.compflags.srcout)) {
-        if (uopts && (uopts->flags & GV_USE_OUTGRAPH))
+        if (uopts->flags & GV_USE_OUTGRAPH)
           addOutputGraph(gs->state, uopts);
         else
           sfioWrite(gs->state->outgraph, gs->opts.outFile);
@@ -1048,6 +1056,11 @@ int gvpr(int argc, char *argv[], gvpropts *uopts) {
   // initialize opts to something that makes freeOpts() a no-op if we fail early
   gvpr_state.opts.outFile = stdout;
 
+  gvpropts DEFAULT_OPTS = {0};
+  if (uopts == NULL) {
+    uopts = &DEFAULT_OPTS;
+  }
+
   int rv = gvpr_core(argc, argv, uopts, &gvpr_state);
 
   // free all allocated resources
